<?php
/**
 * Configuration storage handler classes.
 *
 * Includes following classes user for configuration handling:
 *
 * - class ConfigBase
 *   Base configuration class, other childs must extend this
 *
 * - class ConfigSerializer
 *   Handles configuration data in files
 *
 * - class ConfigDBStorage
 *   Handles configuration data in a table.
 *   NOTE: the table must be created before
 *
 * - class ConfigFactory
 *   Factory class should used to get a instance of a configuration class
 *
 * // TODO: error handling
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @date        18.04.2008
 * @package     Contenido
 * @subpackage  Configuration
 */


/**
 * Base config class
 * 
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigBase {

    var $_bError = false;
    var $_key    = null;
    
    function ConfigBase($options) {
        if (!is_array($options)) {
            die('ConfigBase->ConfigBase: Invalid argument');
        }
        
        if (!isset($options['key'])) {
            die('ConfigBase->ConfigBase: Missing argument $options["key"]');
        }
        
        $this->_key = $options['key'];
    }

    function get(){
        die("ConfigBase->get: Method must be overwritten by child!");
    }

    function set($param){
        die("ConfigBase->set: Method must be overwritten by child!");
    }

    function remove(){
        die("ConfigBase->remove: Method must be overwritten by child!");
    }

}


/**
 * ConfigSerializer to store serialized configuration in files.
 * 
 * Usage:
 * <code>
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('serializer', $options);
 * </code>
 *
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigSerializer extends ConfigBase {

    function ConfigSerializer($options) {
        parent::ConfigBase($options);
    }

    
    function get(){
        if (!is_file($this->_key)) {
            return null;
        }
        $content = file_get_contents($this->_key);
        return unserialize($content);
    }

    
    function set($content){
        // use LOCK_EX if script is running under PHP5
        $flag = (defined('LOCK_EX')) ? LOCK_EX : null;
        $result = file_put_contents($this->_key, serialize($content), $flag);
        return ($result) ? true : false;
    }

    
    function remove(){
        if (is_file($this->_key)) {
            unlink($this->_key);
        }
    }
    
}


/**
 * Class ConfigDBStorage to store serialized configuration in a table.
 *
 * Needs a table, create it if not exists using following statement:
 * <code>
 * -- {prefix} ist almost con by default
 * CREATE TABLE {prefix}_configdbstorage (
 *     `id` varchar(32) NOT NULL default '',
 *     `value` text,
 *     PRIMARY KEY (`id`)
 * )
 * </code>
 * 
 * Usage:
 * <code>
 * // md5 hash of $options['key'] will be used as id in table {prefix}_configdbstorage
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('dbstorage', $options);
 * </code>
 * 
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigDBStorage extends ConfigBase {

    var $_oDB   = null;
    var $_table = '';

    function ConfigDBStorage($options) {
        parent::ConfigBase($options);
        $this->_key   = md5($this->_key);
        $this->_table = $GLOBALS['cfg']['sql']['sqlprefix'] . '_configdbstorage';
        $this->_oDB   = new DB_Contenido();
    }

    
    function get(){
        $sql = 'SELECT value FROM ' . $this->_table . ' WHERE id="' . $this->_key . '"';
        $this->_oDB->query($sql);
        if ($this->_oDB->next_record()) {
            return unserialize($this->_oDB->f('value'));
        } else {
            return null;
        }
    }

    
    function set($content){
        $content = mysql_real_escape_string(serialize($content));
        if ($this->get()) {
            $sql = 'UPDATE ' . $this->_table . ' SET value="' . $content . '" WHERE id="' . $this->_key . '"';
        } else {
            $sql = 'INSERT INTO ' . $this->_table . ' (`id`, `value`) VALUES ("' . $this->_key . '", "' . $content . '")';
        }
        $result = $this->_oDB->query($sql);
        return ($result) ? true : false;

    }

    
    function remove(){
        $sql = 'DELETE FROM ' . $this->_table . ' WHERE id="' . $this->_key . '"';
        $result = $this->_oDB->query($sql);
        return ($result) ? true : false;
    }
    
}



/**
 * Factory class to get an instance of one of configuration classes
 *
 * <code>
 * // ConfigSerializer example
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('serializer', $options);
 *
 * // ConfigDBStorage example
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('dbstorage', $options);
 * </code>
 * 
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigFactory {

    function get($name, $options) {
        if ($name == 'serializer') {
            return new ConfigSerializer($options);
        } elseif ($name == 'dbstorage') {
            return new ConfigDBStorage($options);
        }
        return new ConfigSerializer($options);
        
    }

}


/**
 * Emulate file_put_contents fpr PHP 4
 */
if (!function_exists('file_put_contents') && !defined('FILE_APPEND')) {

    define('FILE_APPEND', 1);
    
    /**
     * This function is identical to calling fopen(), fwrite() and fclose() successively to write data to a file. 
     *
     * @param  string  $filename  Path to the file where to write the data
     * @param  mixed   $data      The data to write. Can be either a string or a array
     * @param  int     $flag      Flag, supported is only FILE_APPEND (=1)
     * @return mixed   Number of written bytes or false on error
     */
    function file_put_contents($filename, $data, $flag=false) {
        $mode = ($flag == FILE_APPEND || strtoupper($flag) == 'FILE_APPEND') ? 'a' : 'w';
        if (!$f = @fopen($filename, $mode)) {
            return false;
        }
        if (is_array($data)) {
            $data = implode($data);
        }
        $bytes_written = fwrite($f, $data);
        fclose($f);
        return $bytes_written;
    }

}
